@@ -266,7 +266,8 @@ exec_data_new (void)
 }
 
 static ExecData *
-exec_data_clone (ExecData *data)
+exec_data_clone (ExecData *data,
+                 gboolean preserve_argv)
 {
   if (data == nullptr)
     return nullptr;
@@ -276,7 +277,8 @@ exec_data_clone (ExecData *data)
   clone->cwd = g_strdup (data->cwd);
 
   /* If FDs were passed, cannot repeat argv. Return data only for env and cwd */
-  if (data->fd_list != nullptr) {
+  if (!preserve_argv ||
+      data->fd_list != nullptr) {
     clone->as_shell = TRUE;
     return clone;
   }
@@ -854,6 +856,7 @@ terminal_screen_reexec_from_screen (TerminalScreen *screen,
 
   g_return_val_if_fail (TERMINAL_IS_SCREEN (parent_screen), FALSE);
 
+  terminal_unref_exec_data ExecData* data = exec_data_clone (parent_screen->priv->exec_data, FALSE);
   gs_free char* cwd = terminal_screen_get_current_dir (parent_screen);
 
   _terminal_debug_print (TERMINAL_DEBUG_PROCESSES,
@@ -863,7 +866,7 @@ terminal_screen_reexec_from_screen (TerminalScreen *screen,
                          cwd);
 
   return terminal_screen_reexec_from_exec_data (screen,
-                                                nullptr /* exec data */,
+                                                data,
                                                 nullptr /* envv */,
                                                 cwd,
                                                 cancellable,
@@ -1582,7 +1585,7 @@ spawn_result_cb (VteTerminal *terminal,
   }
 
   /* Retain info for reexec, if possible */
-  ExecData *new_exec_data = exec_data_clone (exec_data);
+  ExecData *new_exec_data = exec_data_clone (exec_data, TRUE);
   terminal_screen_clear_exec_data (screen, FALSE);
   priv->exec_data = new_exec_data;
   }
